#! /bin/bash

#
#                ╔═══╗╔╗──╔╗╔═══╗╔═╗─╔╗
#                ║╔═╗║║╚╗╔╝║║╔═╗║║║╚╗║║
#                ║║─║║╚╗║║╔╝║╚═╝║║╔╗╚╝║
#                ║║─║║─║╚╝║─║╔══╝║║╚╗║║
#                ║╚═╝║─╚╗╔╝─║║───║║─║║║
#                ╚═══╝──╚╝──╚╝───╚╝─╚═╝
#
# ╔══╗─────────────────╔╗─╔╗     ─────────╔╗──╔╗────────────
# ║╔╗║─────────────────║║─║║     ────────╔╝╚╗╔╝╚╗───────────
# ║╚╝║╔═╗╔╗╔╗╔╗╔══╗╔══╗║╚═╝║     ╔╗╔╗╔══╗╚╗╔╝╚╗╔╝╔══╗╔═╗╔══╗
# ║╔═╝║╔╝─╣║╚╝║║╔╗║║╔═╝╚═╗╔╝     ║╚╝║║╔╗║─║║──║║─║║═╣║╔╝║══╣
# ║║──║║─║║╚╗╔╝║╔╗║║╚═╗╔═╝║─     ║║║║║╔╗║─║╚╗─║╚╗║║═╣║║──══║
# ╚╝──╚╝─╚╝─╚╝─╚╝╚╝╚══╝╚══╝─     ╚╩╩╝╚╝╚╝─╚═╝─╚═╝╚══╝╚╝─╚══╝
#
#   Maintainer: OVPN
#

#-----------------------------------------------------------------------------

if [ "$EUID" -ne 0 ]; then
    echo "This script must be run as root"
    exit 1
fi

# -----------------------------------------------------------------------------
if [ "$#" -ne 6 ];
then
    echo "Usage: $0 {ipv4|ipv6|all} {enable|disable} {killswitch|all} [ovpnserver] [ovpninternalip] [-1]"
    exit 1
fi

# -----------------------------------------------------------------------------
debug=0
proto=$1
action=$2
modules=$3
ovpnserver=$4
ovpninternalip=$5
modules_all='killswitch'
ipt4='iptables'
ipt6='ip6tables'
fwcommentstring="-m comment --comment 'ovpn.com'"
local_subnets="10.0.0.0/8 100.64.0.0/10 172.16.0.0/12 192.168.0.0/16 127.0.0.0/8 224.0.0.0/4"
ipv6_ns1='2a07:a880:4601:10f0:cd45::1'
ipv6_ns2='2001:67c:750:1:cafe:cd45::1'

# -----------------------------------------------------------------------------
ei() {
    if [ "$debug" = 1 ]; then
        echo "$@"
        eval "$@"
    else
        eval "$@" >/dev/null 2>&1
    fi
}

# -----------------------------------------------------------------------------
hook_init() {
    if [ -z "$1" ] || [ -z "$2" ] || [ -z "$3" ] || [ -z "$4" ]; then
        echo
    else
        hook_module=$1
        hook_action=$2
        hook_table=$3
        hook_chains=$4
        for chain in $hook_chains
        do
            ovpn_chain="ovpn_"$hook_module"_"$chain
            ei $ipt -t $hook_table -D $chain -j $ovpn_chain
            ei $ipt -t $hook_table -F $ovpn_chain
            ei $ipt -t $hook_table -X $ovpn_chain
            if [ "$hook_action" = "enable" ];
            then
                ei $ipt -t $hook_table -N $ovpn_chain
                ei $ipt -t $hook_table -I $chain -j $ovpn_chain
            fi
        done
    fi
    
}

# -----------------------------------------------------------------------------
hook_action() {
    if [ -z "$1" ] || [ -z "$2" ]; then
        echo
    else
        hook_module=$1
        hook_action=$2
        hook_init $hook_module $hook_action filter "INPUT FORWARD OUTPUT"
        hook_init $hook_module $hook_action nat "PREROUTING INPUT OUTPUT POSTROUTING"
        echo hook module: $hook_module
        echo hook action: $hook_action
        if [ "$hook_module" == "killswitch" ] && [ "$hook_action" == "enable" ]; then
            echo triggered
            fwoutchain="ovpn_"$hook_module"_OUTPUT"
            if [ "$proto" = "ipv4" ]; then
                # Add all exceptions
                local OVPN_SERVER_AR=($(echo ${ovpnserver} | tr ":" "\n"))
                for ONE_OVPN_SERVER in "${OVPN_SERVER_AR[@]}" ; do
					ei $ipt -A $fwoutchain -d ${ONE_OVPN_SERVER}"/32" $fwcommentstring -j ACCEPT
                done
                
                ei $ipt -A $fwoutchain -o tun+ $fwcommentstring -j ACCEPT
                
                for subnet in $local_subnets
                do
                    ei $ipt -A $fwoutchain -d $subnet $fwcommentstring -j ACCEPT
                done
            elif [ "$proto" = "ipv6" ]; then
                ei $ipt -A $fwoutchain -s fe80::/4 $fwcommentstring -j ACCEPT
                ei $ipt -A $fwoutchain -o tun+ $fwcommentstring -j ACCEPT
            fi
            ei $ipt -A $fwoutchain $fwcommentstring -j DROP
        fi
    fi
}

# -----------------------------------------------------------------------------
if [ "$proto" = "ipv4" ]; then
    ipt=$ipt4
elif [ "$proto" = "ipv6" ]; then
    ipt=$ipt6
elif [ "$proto" = "all" ]; then
    module="killswitch"
    echo "Set action for ipv4"
    proto="ipv4"
    ipt=$ipt4
    echo $proto $module $action
    hook_action $module $action
    
    echo "Set action for ipv6"
    proto="ipv6"
    ipt=$ipt6
    echo $proto $module $action
    hook_action $module $action
    exit 0;
fi

if [ "$modules" = "all" ]; then
    modules=$modules_all
fi

for module in $modules
do
    echo $proto $module $action
    hook_action $module $action
done

# -----------------------------------------------------------------------------
